@@ -6,27 +6,22 @@ use anchor_spl::token;
 use vipers::validate::Validate;
 use vipers::{assert_ata, assert_keys, assert_owner, assert_program};
 
-use crate::AcceptAuthority;
-use crate::ClaimRewards;
-use crate::CreateMiner;
-use crate::CreateQuarry;
-use crate::MutableRewarderWithAuthority;
-use crate::NewRewarder;
-use crate::ReadOnlyRewarderWithAuthority;
-use crate::SetDailyRewards;
-use crate::SetFamine;
-use crate::SetRewardsShare;
-use crate::TransferAuthority;
-use crate::UpdateQuarryRewards;
-use crate::UserStake;
+use crate::addresses;
+use crate::{
+    AcceptAuthority, ClaimRewards, CreateMiner, CreateQuarry, ExtractFees,
+    MutableRewarderWithAuthority, MutableRewarderWithPauseAuthority, NewRewarder,
+    ReadOnlyRewarderWithAuthority, SetAnnualRewards, SetFamine, SetPauseAuthority, SetRewardsShare,
+    TransferAuthority, UpdateQuarryRewards, UserStake,
+};
+
+/// --------------------------------
+/// Rewarder Functions
+/// --------------------------------
 
 impl<'info> Validate<'info> for NewRewarder<'info> {
     fn validate(&self) -> ProgramResult {
         require!(self.base.is_signer, Unauthorized);
 
-        // ensure the claim fee token account is owned by the rewarder
-        // in a future program upgrade, rewarder token accounts will be able
-        // to be flushed to a DAO
         assert_ata!(
             self.claim_fee_token_account,
             self.rewarder,
@@ -34,11 +29,6 @@ impl<'info> Validate<'info> for NewRewarder<'info> {
         );
 
         assert_program!(self.system_program, SYSTEM_PROGRAM_ID);
-        assert_keys!(
-            self.mint_wrapper_program,
-            quarry_mint_wrapper::ID,
-            "mint wrapper"
-        );
         assert_keys!(
             self.mint_wrapper.token_mint,
             self.rewards_token_mint,
@@ -57,9 +47,66 @@ impl<'info> Validate<'info> for NewRewarder<'info> {
     }
 }
 
+impl<'info> Validate<'info> for SetPauseAuthority<'info> {
+    /// Validates the [Rewarder] is correct.
+    fn validate(&self) -> ProgramResult {
+        self.auth.validate()?;
+        require!(!self.auth.rewarder.is_paused, Paused);
+        Ok(())
+    }
+}
+
+impl<'info> Validate<'info> for MutableRewarderWithPauseAuthority<'info> {
+    fn validate(&self) -> ProgramResult {
+        require!(self.pause_authority.is_signer, Unauthorized);
+        assert_keys!(
+            self.rewarder.pause_authority,
+            self.pause_authority,
+            "pause_authority"
+        );
+        Ok(())
+    }
+}
+
+impl<'info> Validate<'info> for TransferAuthority<'info> {
+    /// Validates the [Rewarder] is correct.
+    fn validate(&self) -> ProgramResult {
+        require!(!self.rewarder.is_paused, Paused);
+        require!(self.authority.is_signer, Unauthorized);
+        assert_keys!(self.authority, self.rewarder.authority);
+        Ok(())
+    }
+}
+
+impl<'info> Validate<'info> for AcceptAuthority<'info> {
+    /// Validates the [Rewarder] is correct.
+    fn validate(&self) -> ProgramResult {
+        require!(!self.rewarder.is_paused, Paused);
+        require!(
+            self.rewarder.pending_authority != Pubkey::default(),
+            PendingAuthorityNotSet
+        );
+        Ok(())
+    }
+}
+
+impl<'info> Validate<'info> for SetAnnualRewards<'info> {
+    /// Validates the [Rewarder] is correct.
+    fn validate(&self) -> ProgramResult {
+        self.auth.validate()?;
+        require!(!self.auth.rewarder.is_paused, Paused);
+        Ok(())
+    }
+}
+
+/// --------------------------------
+/// Quarry functions
+/// --------------------------------
+
 impl<'info> Validate<'info> for CreateQuarry<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
+        require!(!self.auth.rewarder.is_paused, Paused);
         assert_program!(self.system_program, SYSTEM_PROGRAM_ID);
         assert_owner!(self.token_mint, token::ID, "token_mint");
         Ok(())
@@ -69,6 +116,7 @@ impl<'info> Validate<'info> for CreateQuarry<'info> {
 impl<'info> Validate<'info> for SetRewardsShare<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
+        require!(!self.auth.rewarder.is_paused, Paused);
         assert_keys!(self.quarry.rewarder_key, self.auth.rewarder, "rewarder");
         Ok(())
     }
@@ -77,17 +125,32 @@ impl<'info> Validate<'info> for SetRewardsShare<'info> {
 impl<'info> Validate<'info> for SetFamine<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
+        require!(!self.auth.rewarder.is_paused, Paused);
         assert_keys!(self.quarry.rewarder_key, self.auth.rewarder, "rewarder");
         Ok(())
     }
 }
 
+impl<'info> Validate<'info> for UpdateQuarryRewards<'info> {
+    fn validate(&self) -> ProgramResult {
+        require!(!self.rewarder.is_paused, Paused);
+        assert_keys!(self.quarry.rewarder_key, self.rewarder, "rewarder");
+        Ok(())
+    }
+}
+
+/// --------------------------------
+/// Miner functions
+/// --------------------------------
+
 impl<'info> Validate<'info> for CreateMiner<'info> {
     fn validate(&self) -> ProgramResult {
+        require!(!self.rewarder.is_paused, Paused);
         assert_ata!(self.miner_vault, self.miner, self.token_mint, "miner vault");
         assert_keys!(self.miner_vault.owner, self.miner, "miner vault owner");
         assert_keys!(self.miner_vault.mint, self.token_mint, "miner vault mint");
         assert_ata!(self.miner_vault, self.miner, self.token_mint, "miner vault");
+        assert_keys!(self.quarry.rewarder_key, self.rewarder, "rewarder");
 
         assert_program!(self.system_program, SYSTEM_PROGRAM_ID);
         assert_program!(self.token_program, TOKEN_PROGRAM_ID);
@@ -103,6 +166,7 @@ impl<'info> Validate<'info> for ClaimRewards<'info> {
     /// Validates a [ClaimRewards] accounts struct.
     fn validate(&self) -> ProgramResult {
         self.stake.validate()?;
+        require!(!self.stake.rewarder.is_paused, Paused);
 
         // mint_wrapper_program validate
         assert_keys!(
@@ -160,49 +224,10 @@ impl<'info> Validate<'info> for ClaimRewards<'info> {
     }
 }
 
-impl<'info> Validate<'info> for TransferAuthority<'info> {
-    /// Validates the [Rewarder] is correct.
-    fn validate(&self) -> ProgramResult {
-        self.rewarder.only_owner(&self.authority)
-    }
-}
-
-impl<'info> Validate<'info> for AcceptAuthority<'info> {
-    /// Validates the [Rewarder] is correct.
-    fn validate(&self) -> ProgramResult {
-        require!(
-            self.rewarder.pending_authority != Pubkey::default(),
-            PendingAuthorityNotSet
-        );
-        Ok(())
-    }
-}
-
-impl<'info> Validate<'info> for SetDailyRewards<'info> {
-    /// Validates the [Rewarder] is correct.
-    fn validate(&self) -> ProgramResult {
-        self.auth.validate()
-    }
-}
-
-impl<'info> Validate<'info> for MutableRewarderWithAuthority<'info> {
-    fn validate(&self) -> ProgramResult {
-        require!(self.authority.is_signer, Unauthorized);
-        assert_keys!(self.rewarder.authority, self.authority, "authority");
-        Ok(())
-    }
-}
-
-impl<'info> Validate<'info> for ReadOnlyRewarderWithAuthority<'info> {
-    /// Validates the [Rewarder] is correct.
-    fn validate(&self) -> ProgramResult {
-        self.rewarder.only_owner(&self.authority)
-    }
-}
-
 impl<'info> Validate<'info> for UserStake<'info> {
     /// Validates the UserStake.
     fn validate(&self) -> ProgramResult {
+        require!(!self.rewarder.is_paused, Paused);
         // authority
         require!(self.authority.is_signer, Unauthorized);
         assert_keys!(self.authority, self.miner.authority, "miner authority");
@@ -238,11 +263,63 @@ impl<'info> Validate<'info> for UserStake<'info> {
     }
 }
 
-impl<'info> Validate<'info> for UpdateQuarryRewards<'info> {
-    /// Validates a [ClaimRewards] accounts struct.
+impl<'info> Validate<'info> for ExtractFees<'info> {
     fn validate(&self) -> ProgramResult {
-        assert_keys!(self.quarry.rewarder_key, self.rewarder, "rewarder");
+        require!(!self.rewarder.is_paused, Paused);
+        assert_ata!(
+            self.claim_fee_token_account,
+            self.rewarder,
+            self.rewarder.rewards_token_mint
+        );
+
+        assert_program!(self.token_program, TOKEN_PROGRAM_ID);
+
+        assert_keys!(
+            self.claim_fee_token_account.mint,
+            self.rewarder.rewards_token_mint,
+            "claim_fee_token_account.mint"
+        );
+        assert_keys!(
+            self.fee_to_token_account.mint,
+            self.rewarder.rewards_token_mint,
+            "fee_to_token_account.mint"
+        );
+        assert_keys!(
+            self.fee_to_token_account.owner,
+            addresses::FEE_TO,
+            "fee_to_token_account.owner"
+        );
+        assert_ata!(
+            self.fee_to_token_account,
+            addresses::FEE_TO,
+            self.rewarder.rewards_token_mint,
+            "fee ata"
+        );
+
+        assert_owner!(
+            self.claim_fee_token_account,
+            token::ID,
+            "claim_fee_token_account"
+        );
+        assert_owner!(self.fee_to_token_account, token::ID, "fee_to_token_account");
 
         Ok(())
     }
 }
+
+impl<'info> Validate<'info> for MutableRewarderWithAuthority<'info> {
+    fn validate(&self) -> ProgramResult {
+        require!(self.authority.is_signer, Unauthorized);
+        assert_keys!(self.rewarder.authority, self.authority, "authority");
+        Ok(())
+    }
+}
+
+impl<'info> Validate<'info> for ReadOnlyRewarderWithAuthority<'info> {
+    /// Validates the [Rewarder] is correct.
+    fn validate(&self) -> ProgramResult {
+        require!(self.authority.is_signer, Unauthorized);
+        assert_keys!(self.authority, self.rewarder.authority);
+        Ok(())
+    }
+}
