@@ -7,36 +7,40 @@
 //! - [Miner], which stake tokens into [Quarry]s to receive rewards.
 //!
 //! This program is modeled after [Synthetix's StakingRewards.sol](https://github.com/Synthetixio/synthetix/blob/4b9b2ee09b38638de6fe1c38dbe4255a11ebed86/contracts/StakingRewards.sol).
-#![allow(clippy::nonstandard_macro_braces)]
+
+#[macro_use]
+mod macros;
 
 use anchor_lang::prelude::*;
-use anchor_lang::solana_program::declare_id;
 use anchor_lang::Key;
 use anchor_spl::token::{self, Mint, TokenAccount, Transfer};
 use num_traits::ToPrimitive;
 use payroll::Payroll;
 use std::cmp;
 use vipers::assert_keys;
-use vipers::program_err;
+use vipers::unwrap_int;
 use vipers::validate::Validate;
 
-mod account_validators;
-mod payroll;
-mod quarry;
-mod rewarder;
+pub mod account_validators;
+pub mod addresses;
+pub mod payroll;
+pub mod quarry;
+pub mod rewarder;
 
-declare_id!("QMNFUvncKBh11ZgEwYtoup3aXvuVxt6fzrcsjk2cjpM");
+use crate::quarry::StakeAction;
 
-/// The fees of new rewarders-- 1,000 KBPS = 1 BP or 0.01%.
-const DEFAULT_CLAIM_FEE_KBPS: u64 = 1_000;
+solana_program::declare_id!("QMNeHCGYnLVDn1icRAfQZpjPLBNkfGbSKRB83G5d8KB");
 
+/// Maximum number of tokens that can be rewarded by a [Rewarder] per year.
+pub const MAX_ANNUAL_REWARDS_RATE: u64 = u64::MAX >> 3;
+
+/// The fees of new [Rewarder]s-- 1,000 KBPS = 1 BP or 0.01%.
+/// This may be changed by governance in the future via program upgrade.
+pub const DEFAULT_CLAIM_FEE_KBPS: u64 = 1_000;
+
+/// Program for [quarry_mine].
 #[program]
 pub mod quarry_mine {
-
-    use vipers::unwrap_int;
-
-    use crate::quarry::StakeAction;
-
     use super::*;
 
     /// --------------------------------
@@ -53,11 +57,10 @@ pub mod quarry_mine {
 
         rewarder.authority = ctx.accounts.authority.key();
         rewarder.pending_authority = Pubkey::default();
-        rewarder.daily_rewards_rate = 0;
+
+        rewarder.annual_rewards_rate = 0;
         rewarder.num_quarries = 0;
         rewarder.total_rewards_shares = 0;
-
-        rewarder.mint_wrapper_program = ctx.accounts.mint_wrapper_program.key();
         rewarder.mint_wrapper = ctx.accounts.mint_wrapper.key();
 
         rewarder.rewards_token_mint = ctx.accounts.rewards_token_mint.key();
@@ -65,6 +68,9 @@ pub mod quarry_mine {
         rewarder.claim_fee_token_account = ctx.accounts.claim_fee_token_account.key();
         rewarder.max_claim_fee_kbps = DEFAULT_CLAIM_FEE_KBPS;
 
+        rewarder.pause_authority = Pubkey::default();
+        rewarder.is_paused = false;
+
         emit!(NewRewarderEvent {
             authority: rewarder.authority,
             timestamp: ctx.accounts.clock.unix_timestamp,
@@ -73,7 +79,31 @@ pub mod quarry_mine {
         Ok(())
     }
 
-    /// Transfers the rewarder authority to a different account.
+    /// Sets the pause authority.
+    #[access_control(ctx.accounts.validate())]
+    pub fn set_pause_authority(ctx: Context<SetPauseAuthority>) -> ProgramResult {
+        let rewarder = &mut ctx.accounts.auth.rewarder;
+        rewarder.pause_authority = ctx.accounts.pause_authority.key();
+        Ok(())
+    }
+
+    /// Pauses the [Rewarder].
+    #[access_control(ctx.accounts.validate())]
+    pub fn pause(ctx: Context<MutableRewarderWithPauseAuthority>) -> ProgramResult {
+        let rewarder = &mut ctx.accounts.rewarder;
+        rewarder.is_paused = true;
+        Ok(())
+    }
+
+    /// Unpauses the [Rewarder].
+    #[access_control(ctx.accounts.validate())]
+    pub fn unpause(ctx: Context<MutableRewarderWithPauseAuthority>) -> ProgramResult {
+        let rewarder = &mut ctx.accounts.rewarder;
+        rewarder.is_paused = false;
+        Ok(())
+    }
+
+    /// Transfers the [Rewarder] authority to a different account.
     #[access_control(ctx.accounts.validate())]
     pub fn transfer_authority(
         ctx: Context<TransferAuthority>,
@@ -97,15 +127,19 @@ pub mod quarry_mine {
 
     /// Sets the amount of reward tokens distributed to all [Quarry]s per day.
     #[access_control(ctx.accounts.validate())]
-    pub fn set_daily_rewards(ctx: Context<SetDailyRewards>, new_rate: u64) -> ProgramResult {
+    pub fn set_annual_rewards(ctx: Context<SetAnnualRewards>, new_rate: u64) -> ProgramResult {
+        require!(
+            new_rate <= MAX_ANNUAL_REWARDS_RATE,
+            MaxAnnualRewardsRateExceeded
+        );
         let rewarder = &mut ctx.accounts.auth.rewarder;
-        let previous_rate = rewarder.daily_rewards_rate;
-        rewarder.daily_rewards_rate = new_rate;
+        let previous_rate = rewarder.annual_rewards_rate;
+        rewarder.annual_rewards_rate = new_rate;
 
-        emit!(RewarderDailyRewardsUpdateEvent {
+        emit!(RewarderAnnualRewardsUpdateEvent {
             previous_rate,
             new_rate,
-            timestamp: ctx.accounts.clock.unix_timestamp as u64,
+            timestamp: ctx.accounts.clock.unix_timestamp,
         });
 
         Ok(())
@@ -116,18 +150,22 @@ pub mod quarry_mine {
     /// --------------------------------
 
     /// Creates a new [Quarry].
+    /// This may only be called by the [Rewarder]::authority.
     #[access_control(ctx.accounts.validate())]
     pub fn create_quarry(ctx: Context<CreateQuarry>, bump: u8) -> ProgramResult {
         let rewarder = &mut ctx.accounts.auth.rewarder;
         // Update rewarder's quarry stats
-        rewarder.num_quarries += 1;
+        let index = rewarder.num_quarries;
+        rewarder.num_quarries = unwrap_int!(rewarder.num_quarries.checked_add(1));
 
         let quarry = &mut ctx.accounts.quarry;
         quarry.bump = bump;
+
         // Set quarry params
+        quarry.index = index;
         quarry.famine_ts = i64::MAX;
         quarry.rewarder_key = *rewarder.to_account_info().key;
-        quarry.daily_rewards_rate = 0;
+        quarry.annual_rewards_rate = 0;
         quarry.rewards_share = 0;
         quarry.token_mint_decimals = ctx.accounts.token_mint.decimals;
         quarry.token_mint_key = *ctx.accounts.token_mint.to_account_info().key;
@@ -145,25 +183,20 @@ pub mod quarry_mine {
     pub fn set_rewards_share(ctx: Context<SetRewardsShare>, new_share: u64) -> ProgramResult {
         let rewarder = &mut ctx.accounts.auth.rewarder;
         let quarry = &mut ctx.accounts.quarry;
-        rewarder.total_rewards_shares =
-            rewarder.total_rewards_shares - quarry.rewards_share + new_share;
-
-        require!(
-            rewarder.validate_quarry_rewards_share(new_share),
-            InvalidRewardsShare
-        );
+        rewarder.total_rewards_shares = unwrap_int!(rewarder
+            .total_rewards_shares
+            .checked_add(new_share)
+            .and_then(|v| v.checked_sub(quarry.rewards_share)));
 
         quarry.last_update_ts = cmp::min(ctx.accounts.clock.unix_timestamp, quarry.famine_ts);
-        quarry.daily_rewards_rate = unwrap_int!(rewarder
-            .compute_quarry_daily_rewards_rate(new_share)
-            .to_u64());
+        quarry.annual_rewards_rate = rewarder.compute_quarry_annual_rewards_rate(new_share)?;
         quarry.rewards_share = new_share;
 
         emit!(QuarryRewardsUpdateEvent {
             token_mint: quarry.token_mint_key,
-            daily_rewards_rate: quarry.daily_rewards_rate,
+            annual_rewards_rate: quarry.annual_rewards_rate,
             rewards_share: quarry.rewards_share,
-            timestamp: ctx.accounts.clock.unix_timestamp as u64,
+            timestamp: ctx.accounts.clock.unix_timestamp,
         });
 
         Ok(())
@@ -190,9 +223,9 @@ pub mod quarry_mine {
 
         emit!(QuarryRewardsUpdateEvent {
             token_mint: quarry.token_mint_key,
-            daily_rewards_rate: quarry.daily_rewards_rate,
+            annual_rewards_rate: quarry.annual_rewards_rate,
             rewards_share: quarry.rewards_share,
-            timestamp: current_ts as u64,
+            timestamp: current_ts,
         });
 
         Ok(())
@@ -202,7 +235,7 @@ pub mod quarry_mine {
     /// Miner functions
     /// --------------------------------
 
-    /// Creates a miner for the given authority.
+    /// Creates a [Miner] for the given authority.
     ///
     /// Anyone can call this; this is an associated account.
     #[access_control(ctx.accounts.validate())]
@@ -225,7 +258,7 @@ pub mod quarry_mine {
         Ok(())
     }
 
-    /// Claims rewards for the miner.
+    /// Claims rewards for the [Miner].
     #[access_control(ctx.accounts.validate())]
     pub fn claim_rewards(ctx: Context<ClaimRewards>) -> ProgramResult {
         let miner = &mut ctx.accounts.stake.miner;
@@ -259,11 +292,7 @@ pub mod quarry_mine {
         let amount_claimable_minus_fees = unwrap_int!(amount_claimable.checked_sub(max_claim_fee));
 
         // Create the signer seeds.
-        let seeds = &[
-            b"Rewarder".as_ref(),
-            ctx.accounts.stake.rewarder.base.as_ref(),
-            &[ctx.accounts.stake.rewarder.bump],
-        ];
+        let seeds = gen_rewarder_signer_seeds!(ctx.accounts.stake.rewarder);
         let signer_seeds = &[&seeds[..]];
 
         // Mint the claimed tokens.
@@ -317,6 +346,7 @@ pub mod quarry_mine {
         Ok(())
     }
 
+    /// Stakes tokens into the [Miner].
     #[access_control(ctx.accounts.validate())]
     pub fn stake_tokens(ctx: Context<UserStake>, amount: u64) -> ProgramResult {
         if amount == 0 {
@@ -353,6 +383,7 @@ pub mod quarry_mine {
         Ok(())
     }
 
+    /// Withdraws tokens from the [Miner].
     #[access_control(ctx.accounts.validate())]
     pub fn withdraw_tokens(ctx: Context<UserStake>, amount: u64) -> ProgramResult {
         if amount == 0 {
@@ -404,9 +435,32 @@ pub mod quarry_mine {
         Ok(())
     }
 
-    /// Stub for generating the ClaimRewards helpers on the frontend.
-    pub fn _unused_stub_claim_rewards(_ctx: Context<ClaimRewards>) -> ProgramResult {
-        program_err!(Unauthorized)
+    /// --------------------------------
+    /// Protocol Functions
+    /// --------------------------------
+
+    /// Extracts fees to the Quarry DAO.
+    /// This can be called by anyone.
+    #[access_control(ctx.accounts.validate())]
+    pub fn extract_fees(ctx: Context<ExtractFees>) -> ProgramResult {
+        let seeds = gen_rewarder_signer_seeds!(ctx.accounts.rewarder);
+        let signer_seeds = &[&seeds[..]];
+
+        // Transfer the tokens to the DAO address.
+        token::transfer(
+            CpiContext::new_with_signer(
+                ctx.accounts.token_program.clone(),
+                token::Transfer {
+                    from: ctx.accounts.claim_fee_token_account.to_account_info(),
+                    to: ctx.accounts.fee_to_token_account.to_account_info(),
+                    authority: ctx.accounts.rewarder.to_account_info(),
+                },
+                signer_seeds,
+            ),
+            ctx.accounts.claim_fee_token_account.amount,
+        )?;
+
+        Ok(())
     }
 }
 
@@ -428,14 +482,15 @@ pub struct Rewarder {
     pub authority: Pubkey,
     /// Pending authority which must accept the authority
     pub pending_authority: Pubkey,
-    /// Number of quarries the rewarder manages
+
+    /// Number of [Quarry]s the [Rewarder] manages.
+    /// If more than this many [Quarry]s are desired, one can create
+    /// a second rewarder.
     pub num_quarries: u16,
     /// Amount of reward tokens distributed per day
-    pub daily_rewards_rate: u64,
-    /// Total amount of rewards shares allocated to quarries
+    pub annual_rewards_rate: u64,
+    /// Total amount of rewards shares allocated to [Quarry]s
     pub total_rewards_shares: u64,
-    /// Mint wrapper program that the rewarder will pull from.
-    pub mint_wrapper_program: Pubkey,
     /// Mint wrapper.
     pub mint_wrapper: Pubkey,
     /// Mint of the rewards token for this [Rewarder].
@@ -448,6 +503,11 @@ pub struct Rewarder {
     /// This is stored on the [Rewarder] to ensure that the fee will
     /// not exceed this in the future.
     pub max_claim_fee_kbps: u64,
+
+    /// Authority allowed to pause a [Rewarder].
+    pub pause_authority: Pubkey,
+    /// If true, all instructions on the [Rewarder] are paused other than [quarry_mine::unpause].
+    pub is_paused: bool,
 }
 
 /// A pool which distributes tokens to its [Miner]s.
@@ -461,16 +521,18 @@ pub struct Quarry {
     /// Bump.
     pub bump: u8,
 
-    /// Decimals on the token mint
+    /// Index of the [Quarry].
+    pub index: u16,
+    /// Decimals on the token [Mint].
     pub token_mint_decimals: u8,
     /// Timestamp when quarry rewards cease
     pub famine_ts: i64,
     /// Timestamp of last checkpoint
     pub last_update_ts: i64,
     /// Rewards per token stored in the quarry
-    pub rewards_per_token_stored: u64,
-    /// Rewards rate for this quarry
-    pub daily_rewards_rate: u64,
+    pub rewards_per_token_stored: u128,
+    /// Amount of rewards distributed to the quarry per year.
+    pub annual_rewards_rate: u64,
     /// Rewards shared allocated to this quarry
     pub rewards_share: u64,
 
@@ -480,7 +542,7 @@ pub struct Quarry {
 
 /// An account that has staked tokens into a [Quarry].
 #[account]
-#[derive(Default)]
+#[derive(Default, Debug)]
 pub struct Miner {
     /// Key of the [Quarry] this [Miner] works on.
     pub quarry_key: Pubkey,
@@ -498,15 +560,15 @@ pub struct Miner {
     /// Whenever the [Miner] claims tokens, this is reset to 0.
     pub rewards_earned: u64,
 
-    /// A checkpoint of the quarry's reward tokens paid per staked token.
+    /// A checkpoint of the [Quarry]'s reward tokens paid per staked token.
     ///
     /// When the [Miner] is initialized, this number starts at 0.
-    /// On the first [farm::stake_tokens], the [Quarry::update_rewards_and_miner]
+    /// On the first [quarry_mine::stake_tokens], the [Quarry]#update_rewards_and_miner
     /// method is called, which updates this checkpoint to the current quarry value.
     ///
-    /// On a [farm::claim_rewards], the difference in checkpoints is used to calculate
+    /// On a [quarry_mine::claim_rewards], the difference in checkpoints is used to calculate
     /// the amount of tokens owed.
-    pub rewards_per_token_paid: u64,
+    pub rewards_per_token_paid: u128,
 
     /// Number of tokens the [Miner] holds.
     pub balance: u64,
@@ -517,6 +579,8 @@ pub struct Miner {
 /// --------------------------------
 
 /* Rewarder contexts */
+
+/// Accounts for [quarry_mine::new_rewarder].
 #[derive(Accounts)]
 #[instruction(bump: u8)]
 pub struct NewRewarder<'info> {
@@ -549,9 +613,6 @@ pub struct NewRewarder<'info> {
     /// Clock.
     pub clock: Sysvar<'info, Clock>,
 
-    /// Mint wrapper program.
-    pub mint_wrapper_program: AccountInfo<'info>,
-
     /// Mint wrapper.
     pub mint_wrapper: CpiAccount<'info, quarry_mint_wrapper::MintWrapper>,
 
@@ -562,6 +623,17 @@ pub struct NewRewarder<'info> {
     pub claim_fee_token_account: CpiAccount<'info, TokenAccount>,
 }
 
+/// Accounts for [quarry_mine::set_pause_authority].
+#[derive(Accounts)]
+pub struct SetPauseAuthority<'info> {
+    /// [Rewarder].
+    pub auth: MutableRewarderWithAuthority<'info>,
+
+    /// The pause authority.
+    pub pause_authority: AccountInfo<'info>,
+}
+
+/// Accounts for [quarry_mine::transfer_authority].
 #[derive(Accounts)]
 pub struct TransferAuthority<'info> {
     /// Authority of the rewarder.
@@ -573,6 +645,7 @@ pub struct TransferAuthority<'info> {
     pub rewarder: ProgramAccount<'info, Rewarder>,
 }
 
+/// Accounts for [quarry_mine::accept_authority].
 #[derive(Accounts)]
 pub struct AcceptAuthority<'info> {
     /// Authority of the next rewarder.
@@ -584,6 +657,7 @@ pub struct AcceptAuthority<'info> {
     pub rewarder: ProgramAccount<'info, Rewarder>,
 }
 
+/// Mutable [Rewarder] that requires the authority to be a signer.
 #[derive(Accounts)]
 pub struct MutableRewarderWithAuthority<'info> {
     /// Authority of the rewarder.
@@ -595,6 +669,7 @@ pub struct MutableRewarderWithAuthority<'info> {
     pub rewarder: ProgramAccount<'info, Rewarder>,
 }
 
+/// Read-only [Rewarder] that requires the authority to be a signer.
 #[derive(Accounts)]
 pub struct ReadOnlyRewarderWithAuthority<'info> {
     /// Authority of the rewarder.
@@ -605,18 +680,23 @@ pub struct ReadOnlyRewarderWithAuthority<'info> {
     pub rewarder: ProgramAccount<'info, Rewarder>,
 }
 
+/// Accounts for [quarry_mine::set_annual_rewards].
 #[derive(Accounts)]
-pub struct SetDailyRewards<'info> {
+pub struct SetAnnualRewards<'info> {
+    /// [Rewarder],
     pub auth: MutableRewarderWithAuthority<'info>,
+
+    /// [Clock].
     pub clock: Sysvar<'info, Clock>,
 }
 
 /* Quarry contexts */
 
+/// Accounts for [quarry_mine::create_quarry].
 #[derive(Accounts)]
 #[instruction(bump: u8)]
 pub struct CreateQuarry<'info> {
-    /// Quarry.
+    /// [Quarry].
     #[account(
         init,
         seeds = [
@@ -629,67 +709,72 @@ pub struct CreateQuarry<'info> {
     )]
     pub quarry: ProgramAccount<'info, Quarry>,
 
-    /// Rewarder authority.
+    /// [Rewarder] authority.
     pub auth: MutableRewarderWithAuthority<'info>,
 
-    /// Mint of the token to create a quarry for.
+    /// [Mint] of the token to create a [Quarry] for.
     pub token_mint: CpiAccount<'info, Mint>,
 
-    /// Payer of quarry creation.
+    /// Payer of [Quarry] creation.
     pub payer: AccountInfo<'info>,
 
-    /// Clock.
+    /// [Clock].
     pub clock: Sysvar<'info, Clock>,
 
     /// System program.
     pub system_program: AccountInfo<'info>,
 }
 
+/// Accounts for [quarry_mine::set_famine].
 #[derive(Accounts)]
 pub struct SetFamine<'info> {
-    /// Rewarder of the quarry.
+    /// [Rewarder] of the [Quarry].
     pub auth: ReadOnlyRewarderWithAuthority<'info>,
 
-    /// Quarry updated.
+    /// [Quarry] updated.
     #[account(mut)]
     pub quarry: ProgramAccount<'info, Quarry>,
 }
 
+/// Accounts for [quarry_mine::set_rewards_share].
 #[derive(Accounts)]
 pub struct SetRewardsShare<'info> {
-    /// Rewarder of the quarry.
+    /// [Rewarder] of the [Quarry].
     pub auth: MutableRewarderWithAuthority<'info>,
 
-    /// Quarry updated.
+    /// [Quarry] updated.
     #[account(mut)]
     pub quarry: ProgramAccount<'info, Quarry>,
 
-    /// Clock.
+    /// [Clock].
     pub clock: Sysvar<'info, Clock>,
 }
 
+/// Accounts for [quarry_mine::update_quarry_rewards].
 #[derive(Accounts)]
 pub struct UpdateQuarryRewards<'info> {
-    /// Quarry.
+    /// [Quarry].
     #[account(mut)]
     pub quarry: ProgramAccount<'info, Quarry>,
 
-    /// Rewarder
+    /// [Rewarder].
     pub rewarder: ProgramAccount<'info, Rewarder>,
 
-    /// Clock.
+    /// [Clock].
     pub clock: Sysvar<'info, Clock>,
 }
 
 /* Miner contexts */
+
+/// Accounts for [quarry_mine::create_miner].
 #[derive(Accounts)]
 #[instruction(bump: u8)]
 pub struct CreateMiner<'info> {
-    /// Authority of the miner.
+    /// Authority of the [Miner].
     #[account(signer)]
     pub authority: AccountInfo<'info>,
 
-    /// Miner to be created.
+    /// [Miner] to be created.
     #[account(
         init,
         seeds = [
@@ -705,16 +790,19 @@ pub struct CreateMiner<'info> {
     /// [Quarry] to create a [Miner] for.
     pub quarry: ProgramAccount<'info, Quarry>,
 
-    /// System program
+    /// [Rewarder].
+    pub rewarder: ProgramAccount<'info, Rewarder>,
+
+    /// System program.
     pub system_program: AccountInfo<'info>,
 
-    /// Payer of miner creation.
+    /// Payer of [Miner] creation.
     pub payer: AccountInfo<'info>,
 
-    /// Mint of the token to create a quarry for.
+    /// [Mint] of the token to create a [Quarry] for.
     pub token_mint: CpiAccount<'info, Mint>,
 
-    /// Token account holding the token mint.
+    /// [TokenAccount] holding the token [Mint].
     pub miner_vault: CpiAccount<'info, TokenAccount>,
 
     /// SPL Token program.
@@ -725,6 +813,7 @@ pub struct CreateMiner<'info> {
 #[derive(Accounts)]
 pub struct ClaimRewards<'info> {
     /// Mint wrapper.
+    #[account(mut)]
     pub mint_wrapper: CpiAccount<'info, quarry_mint_wrapper::MintWrapper>,
     /// Mint wrapper program.
     pub mint_wrapper_program: AccountInfo<'info>,
@@ -786,6 +875,37 @@ pub struct UserStake<'info> {
     pub clock: Sysvar<'info, Clock>,
 }
 
+/// Accounts for [quarry_mine::extract_fees].
+#[derive(Accounts)]
+pub struct ExtractFees<'info> {
+    /// Rewarder to extract fees from.
+    pub rewarder: ProgramAccount<'info, Rewarder>,
+
+    /// [TokenAccount] which receives claim fees.
+    #[account(mut)]
+    pub claim_fee_token_account: CpiAccount<'info, TokenAccount>,
+
+    /// [TokenAccount] owned by the [addresses::FEE_TO_ADDRESS].
+    /// Holds DAO claim fees.
+    #[account(mut)]
+    pub fee_to_token_account: CpiAccount<'info, TokenAccount>,
+
+    /// Token program
+    pub token_program: AccountInfo<'info>,
+}
+
+/// Accounts for [quarry_mine::pause] and [quarry_mine::unpause].
+#[derive(Accounts)]
+pub struct MutableRewarderWithPauseAuthority<'info> {
+    /// Pause authority of the rewarder.
+    #[account(signer)]
+    pub pause_authority: AccountInfo<'info>,
+
+    /// Rewarder of the farm.
+    #[account(mut)]
+    pub rewarder: ProgramAccount<'info, Rewarder>,
+}
+
 /// --------------------------------
 /// Events
 /// --------------------------------
@@ -851,10 +971,13 @@ pub struct WithdrawEvent {
 
 /// Triggered when the daily rewards rate is updated.
 #[event]
-pub struct RewarderDailyRewardsUpdateEvent {
-    previous_rate: u64,
-    new_rate: u64,
-    timestamp: u64,
+pub struct RewarderAnnualRewardsUpdateEvent {
+    /// Previous rate of rewards.
+    pub previous_rate: u64,
+    /// New rate of rewards.
+    pub new_rate: u64,
+    /// When the event took place.
+    pub timestamp: i64,
 }
 
 /// Triggered when a new miner is created.
@@ -863,25 +986,33 @@ pub struct MinerCreateEvent {
     /// Authority of the miner.
     #[index]
     pub authority: Pubkey,
+    /// Quarry the miner was created on.
     #[index]
     pub quarry: Pubkey,
+    /// The [Miner].
     pub miner: Pubkey,
 }
 
 /// Triggered when a new quarry is created.
 #[event]
 pub struct QuarryCreateEvent {
+    /// [Mint] of the [Quarry] token.
     pub token_mint: Pubkey,
+    /// When the event took place.
     pub timestamp: i64,
 }
 
 /// Triggered when a quarry's reward share is updated.
 #[event]
 pub struct QuarryRewardsUpdateEvent {
-    token_mint: Pubkey,
-    daily_rewards_rate: u64,
-    rewards_share: u64,
-    timestamp: u64,
+    /// [Mint] of the [Quarry] token.
+    pub token_mint: Pubkey,
+    /// New annual rewards rate
+    pub annual_rewards_rate: u64,
+    /// New rewards share.
+    pub rewards_share: u64,
+    /// When the event took place.
+    pub timestamp: i64,
 }
 
 /// --------------------------------
@@ -907,4 +1038,10 @@ pub enum ErrorCode {
     InvalidTimestamp,
     #[msg("Invalid max claim fee.")]
     InvalidMaxClaimFee,
+    #[msg("Max annual rewards rate exceeded.")]
+    MaxAnnualRewardsRateExceeded,
+    #[msg("Rewarder is paused.")]
+    Paused,
+    #[msg("Rewards earned exceeded quarry's upper bound.")]
+    UpperboundExceeded,
 }
